local addon = Poppins
if not addon then return end
local module = addon:NewModule("ButtonFacadeSkins", "AceHook-3.0")

local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

local LibButtonFacade
local LibStickyFrames = LibStub("LibStickyFrames-2.0")

local function get(key)
	local option = key[#key]
	return module.db.profile[option]
end
local options = {
	name = module.moduleName,
	desc = module.moduleName,
	type = "group",
	handler = module,
	args = {
		enable = {
			name = "Enable",
			desc = "Enables / disables the module",
			type = "toggle",
			get = "IsEnabled",
			set = function(key, value)
				if value then
					module:Enable()
				else
					module:Disable()
				end
			end,
		},
		new = {
			name = "New Skin",
			desc = "Adds a skin",
			type = "input",
			set = "AddSkin",
		},
		remove = {
			name = "Remove skin",
			desc = "Removes a skin",
			type = "select",
			set = "RemoveSkin",
			values = {},
			disabled = function()
				return not next(module.db.profile.skins)
			end,
		},
		skins = {
			name = "Skins",
			desc = "Set insets for ButtonFacade skins",
			type = "group",
			args = {
			},
		},
	},
}
local defaultDB = {
	profile = {
		enabled = true,
		skins = {
			["*"] = { 0, 0, 0, 0 },
			Blizzard = { -3, -2, -2, -3 },
			Apathy = { 3, 3, 3, 3 }
		}
	}
}

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("ButtonFacadeSkins", defaultDB)
	self.db.RegisterCallback(module, "OnProfileChanged")
	self.db.RegisterCallback(module, "OnProfileReset", "OnProfileChanged")
	self:SetEnabledState(self.db.profile.enabled)

	addon.options.args.modules.args[module.moduleName] = options
end

function module:OnEnable()
	self.db.profile.enabled = true
	addon.Frame.prototype.RegisterCallback(self, "OnFrameSetup")
	addon.PopButton.prototype.RegisterCallback(self, "OnPopButtonSetup")
	LibButtonFacade = LibButtonFacade or LibStub("LibButtonFacade", true)
	self:SecureHook(addon, "LibButtonFacade_Update", nil) --module.LibButtonFacade_Update)
	for skin in pairs(self.db.profile.skins) do
		self:AddSkin(skin)
	end
	--ok we seem to late for the first LibButtonFacade_Update and do not want to move it to OnInitialize so we get the skins ourselves
	for id, button in ipairs(addon.buttons) do
		self:AddSkin(button.db.skin)
		local group = button.db.group
		if group then
			local groupskin = self:GetGroupSkin(group)
			self:AddSkin(groupskin)
		end
	end
	self:UpdateAllInsets()
end

function module:OnDisable()
	self.db.profile.enabled = false
	self:UpdateAllInsets()
	self:UnhookAll()
end

module.OnProfileChanged = addon.OnProfileChanged

function module:OnFrameSetup(event, frame, id, db)
	local group = db.group or tostring(id)
	local skin = self:GetGroupSkin(group)
	self:UpdateButtonInsets(frame, skin)
end

function module:OnPopButtonSetup(event, popbutton, id, parent, db)
--	local group = db.group or tostring(id)
--	db.group only consists of spell and type for PopButtons, we need to use parent.db.group here
	local group = parent.db.group or tostring(id)
	local skin = self:GetGroupSkin(group)
	self:UpdateButtonInsets(popbutton, skin)
end

function module:LibButtonFacade_Update(skin, gloss, backdrop, group)
	module:AddSkin(skin)
	module:UpdateGroupInsets(skin, group)
end

function module:GetGroupSkin(group)
	return self.db.profile.enabled and LibButtonFacade and LibButtonFacade:Group(addon.name, group).SkinID or "Blizzard"
end

function module:UpdateAllInsets()
	for id, button in ipairs(addon.buttons) do
		local group = button.db.group or tostring(id)
--		local group = addon.db.profile[id].group or tostring(id)
		local skin = self:GetGroupSkin(group)
		self:UpdateButtonInsets(button, skin)
	end
end

function module:UpdateGroupInsets(skin, group)
	if not group or group == "nil" then return end
	local group = tonumber(group) or group

	for id, button in ipairs(addon.buttons) do
	-- fails for db (nil)
--		if id == group or button.db.group == group then
		if id == group or addon.db.profile[id].group == group then
			self:UpdateButtonInsets(button, skin)
		end
	end
end

function module:ApplySkinInsets(skin)
	for id, button in ipairs(addon.buttons) do
		local group = button.db.group or tostring(id)
		local skinID = self:GetGroupSkin(group)
		if skinID == skin then
			self:UpdateButtonInsets(button, skin)
		end
	end
end

function module:UpdateButtonInsets(button, skin)
	--quick and dirty hack incoming 
	--reason: If there is a BF skin with spaces users will not get into the options menu at all (Validate failed because key contains space)
	--since "DreamLayout" is "Dream Layout" now, isup that are basically all BF users, hopefully Sorata will provide a real solution :)
	--Dont like this kind of fixes, but since the option menu totally fails otherwise even with the module disabled...well dont kill me please :)
	-- part 1\3 of quick an dirty hack
	skin = gsub(skin, " ", "_")
	--quick and dirty spaces hack 1\3 end
	local l, t, r, b = unpack(self.db.profile.skins[skin])
	LibStickyFrames:SetFrameInsets(button, l, t, r, b)
	-- this if is because this function is called with popbuttons as (button,skin) from within OnPopButtonSetup
	if button.popbuttons then
		for id, popbutton in ipairs(button.popbuttons) do
			LibStickyFrames:SetFrameInsets(popbutton, l, t, r, b)
		end
	end
end

do
	local function set(key, value)
		local skin = key[#key-1]
		local option = key.arg
		local db = module.db.profile.skins[skin]
		db[option] = value
		-- part 2\3 of quick an dirty spaces hack
		skin = gsub(skin, "_", " ")
		--quick and dirty spaces hack 2\3 end
		module:ApplySkinInsets(skin)
	end
	local function get(key)
		local skin = key[#key-1]
		local option = key.arg
		return module.db.profile.skins[skin][option]
	end
	function module:AddSkin(key, skin)
		if type(key) == "string" then
			skin = key
		end
		if not skin then return end
		--part 3\3 of quick and dirty spaces hack
		skin = gsub(skin, " ", "_")
		--quick and dirty spaces hack 3\3 end
		options.args.skins.args[skin] = options.args.skins.args[skin] or {
			name = skin,
			desc = ("%s options"):format(skin),
			type = "group",
			args = {
				left = {
					name = "Left Inset",
					desc = "Sets the left inset",
					type = "range",
					min = -36,
					max = 36,
					bigStep = 1,
					get = get,
					set = set,
					arg = 1,
					order = 1,
				},
				top = {
					name = "Top Inset",
					desc = "Sets the top inset",
					type = "range",
					min = -36,
					max = 36,
					bigStep = 1,
					get = get,
					set = set,
					arg = 2,
					order = 2,
				},
				right = {
					name = "Right Inset",
					desc = "Sets the right inset",
					type = "range",
					min = -36,
					max = 36,
					bigStep = 1,
					get = get,
					set = set,
					arg = 3,
					order = 3,
				},
				bottom = {
					name = "Bottom Inset",
					desc = "Sets the bottom inset",
					type = "range",
					min = -36,
					max = 36,
					bigStep = 1,
					get = get,
					set = set,
					arg = 4,
					order = 4,
				},
			},
		}
		options.args.skins.args[skin].hidden = false
		options.args.remove.values[skin] = skin
		AceConfigRegistry:NotifyChange(addon.name)
	end
end

function module:RemoveSkin(key, skin)
	if type(key) == "string" then
		skin = key
	end
	if not skin then return end
	addon.db.profile.skins[skin] = nil
	options.args.skins.args[skin].hidden = true
	groupValues[group] = nil
	options.args.remove.values[skin] = nil
	AceConfigRegistry:NotifyChange(addon.name)
end
--hooksecurefunc(addon, "LibButtonFacade_Update", module.LibButtonFacade_Update)
