--[[
Name: LibSpellsBuff-1.0
Revision: $Rev: 79058 $
Author(s): philosound
Website: http://www.wowace.com/wiki/LibSpellsBuff-1.0
Documentation: http://www.wowace.com/wiki/LibSpellsBuff-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibSpellsBuff-1.0
Description: Provides information about Buff-Spells the classes/player can cast
--]]

local MAJOR_VERSION = "LibSpellsBuff-1.0"
local MINOR_VERSION = tonumber(("$Revision: 79058 $"):match("%d+"))

--[[
	LibSpellsBuff-1.0
		by philosound
		Totems are EXCLUDED.
		Functions:
		GetSelfBuffSpells() - returns a table with the Buffs the player can cast only on himself
		IsSelfBuffSpell(spellId or spellName or spellLink)
--]]

local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end
---------------------
-- Event Frame --
---------------------
--[[
lib.EventFrame = lib.EventFrame or CreateFrame("Frame");
lib.EventFrame:SetScript("OnEvent", function (this, event, ...) lib[event](lib, ...) end)
lib.EventFrame:UnregisterAllEvents();
lib.EventFrame:RegisterEvent("PLAYER_LOGIN");
lib.EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
--]]
---------------------------------------
-- Embed CallbackHandler-1.0 --
---------------------------------------

-- lib.Callbacks = LibStub("CallbackHandler-1.0"):New(lib)

-----------------------------------------
-- Frequently Accessed Globals --
-----------------------------------------

local _GetSpellInfo = GetSpellInfo
local _GetPlayerBuffTexture = GetPlayerBuffTexture
local _GetPlayerBuff = GetPlayerBuff
local _GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local _GetPlayerBuffName = GetPlayerBuffName
local _GetPlayerBuffApplications = GetPlayerBuffApplications
local _GetPlayerBuffDispelType = GetPlayerBuffDispelType
local _GetPlayerBuffTexture = GetPlayerBuffTexture

local table_insert = table.insert

------------------------
-- local functions --
------------------------

local function SpellName(spellID)
	local name = _GetSpellInfo(spellID)
	return name
end

local function GetImpTable()
	local imp =
	{
		[SpellName(2947)] = "self",	-- fire shield
	}
	return imp
end

local function GetClassTable(class)
	local classTable = {}
	if class == "DRUID" then 
		classTable = {
			[SpellName(16689)]= {dur = 45, tags = {"self"}}, 			-- Nature's Grasp
			[SpellName(22812)]= {dur = 12, tags = {"self"}}, 			-- Barkskin
			[SpellName(16864)]= {dur = 1800, tags = {"self"}}, 			-- Omen of Clarity
			
			
			[SpellName(467)]= {dur = 600, tags = {"target"}}, 			-- Thorns
			[SpellName(29166)]= {dur = 20, tags = {"target"}},			-- Innervate
			[SpellName(17116)]= {dur = -1, tags = {"target"}}, 			-- Nature's Swiftness
			[SpellName(1126)]= {dur = 1800, tags = {"target"}}, 		-- Mark of the Wild
			[SpellName(21849)]= {dur = 3600, tags = {"targetparty"}},	-- Gift of the Wild

		}
	elseif class == "HUNTER" then
			-- only 1 aspect at a time, aspect on self is like a stance since it lasts till death or change
			-- party->aura
		classTable = {
			[SpellName(13163)]= {dur = -1, tags = {"self", "aspect"}}, 	-- Aspect of the Monkey
			[SpellName(13165)]= {dur = -1, tags = {"self", "aspect"}}, 	-- Aspect of the Hawk
			[SpellName(5118)]= {dur = -1, tags = {"self", "aspect"}},	-- Aspect of the Cheetah
			[SpellName(13161)]= {dur = -1, tags = {"self", "aspect"}}, 	-- Aspect of the Beast
			[SpellName(34074)]= {dur = -1, tags = {"self", "aspect"}}, 	-- Aspect of the Viper
			
			[SpellName(13159)]= {dur = -1, tags = {"party", "aspect"}},	-- Aspect of the Pack
			[SpellName(20043)]= {dur = -1, tags = {"party", "aspect"}},	-- Aspect of the Wild
			
			[SpellName(19506)]= {dur = -1, tags = {"party"}},			-- Trueshot Aura
		}
	elseif class == "MAGE" then 
		classTable = {
			[SpellName(130)]= {dur = 30, tags = {"self", "short"}},				-- Slow Fall
			[SpellName(12043)]= {dur = -1, tags = {"self", "short"}},			-- Presence of Mind
			[SpellName(12042)]= {dur = 20, tags = {"self", "short"}},			-- Arcane Power
			[SpellName(12051)]= {dur = 8, tags = {"self", "short"}},			-- Evocation
			[SpellName(66)]= {dur = 25, tags = {"self", "short"}},				-- Invisibility 5 till start then 20...this will need some work
			[SpellName(11129)]= {dur = -1, tags = {"self", "short"}},			-- Combustion
			[SpellName(12472)]= {dur = 20, tags = {"self", "short"}},			-- Icy Veins
			[SpellName(45438)]= {dur = 10, tags = {"self", "short"}},			-- Ice Block
			[SpellName(11426)]= {dur = 60, tags = {"self", "short"}},			--Ice Barrier
			
			[SpellName(543)]= {dur = 30, tags = {"self", "short", "ward"}},		-- Fire Ward
			[SpellName(6143)]= {dur = 30, tags = {"self", "short", "ward"}},	-- Frost Ward
			
			[SpellName(1463)]= {dur = 60, tags = {"self", "shield"}},			-- mana shield
--			[SpellName(11252)]= {dur = 60, tags = {"self", "shield"}},		-- improved mana shield
--			[SpellName(11094)]= {tags = {"self", "shield"}},		-- molten shields

			[SpellName(6117)]= {dur = 1800, tags = {"self", "long", "armor"}},	-- Mage Armor
			[SpellName(30482)]= {dur = 1800, tags = {"self", "long", "armor"}},	-- Molten Armor
			[SpellName(168)]= {dur = 1800, tags = {"self", "long", "armor"}},	-- Frost Armor
			[SpellName(7302)]= {dur = 1800, tags = {"self", "long", "armor"}},	-- Ice Armor, stupid blizz
			
			[SpellName(604)]= {dur = 600, tags = {"target"}},					-- Dampen Magic
			[SpellName(1008)]= {dur = 600, tags = {"target"}},					-- Amplify Magic
			
			[SpellName(1459)]= {dur = 1800, tags = {"target"}},					-- Arcane Intellect
			[SpellName(23028)]= {dur = 3600, tags = {"targetparty"}},			-- Arcane Brilliance
		}
	elseif class == "PALADIN" then
		classTable = {
			-- holy trainer
			[SpellName(21084)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Righteousness
			[SpellName(20165)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Light
			[SpellName(20166)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Wisdom
			
			[SpellName(19742)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Wisdom
			[SpellName(19977)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Light
			
			[SpellName(20216)]= {dur = -1, tags = {"self"}},						-- Divine Favor
			[SpellName(31842)]= {dur = 15, tags = {"self"}},						-- Divine Illumination
			
			[SpellName(25894)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Wisdom
			[SpellName(25890)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Light
			
			-- protection trainer
			[SpellName(20164)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Justice
			
			[SpellName(465)]= {dur = -1, tags = {"party", "aura"}},					-- Devotion Aura
			[SpellName(19746)]= {dur = -1, tags = {"party", "aura"}},				-- Concentration Aura
			[SpellName(19876)]= {dur = -1, tags = {"party", "aura"}},				-- Shadow Resistance Aura
			[SpellName(19888)]= {dur = -1, tags = {"party", "aura"}},				-- Frost Resistance Aura
			[SpellName(19891)]= {dur = -1, tags = {"party", "aura"}},				-- Fire Resistance Aura
			
			[SpellName(19742)]= {dur = 6, tags = {"target", "blessing"}},			-- Blessing of Protection
			[SpellName(1044)]= {dur = 10, tags = {"target", "blessing"}},			-- Blessing of Freedom
			[SpellName(1044)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Kings
			[SpellName(1038)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Salvation
			[SpellName(20911)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Sanctuary
			[SpellName(6940)]= {dur = 30, tags = {"target", "blessing"}},			-- Blessing of Sacrifice
			
			[SpellName(498)]= {dur = 6, tags = {"self"}},							-- Divine Protection
			[SpellName(642)]= {dur = 10, tags = {"self"}},							-- Divine Shield
			[SpellName(20925)]= {dur = 10, tags = {"self"}},						-- Holy Shield
			
			[SpellName(25898)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Kings
			[SpellName(25895)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Salvation
			[SpellName(25899)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Sanctuary
			
			-- retribution trainer
			[SpellName(21082)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of the Crusader
			[SpellName(20375)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Command
			[SpellName(31892)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Blood Blood elf
			[SpellName(31801)]= {dur = 30, tags = {"self", "seal"}},				-- Seal of Vengeance Human, Dwarf, Draenei
			
			[SpellName(7294)]= {dur = -1, tags = {"party", "aura"}},				-- Retribution Aura
			[SpellName(20218)]= {dur = -1, tags = {"party", "aura"}},				-- Sanctity Aura
			[SpellName(32223)]= {dur = -1, tags = {"party", "aura"}},				-- Crusader Aura
			
			[SpellName(19740)]= {dur = 600, tags = {"target", "blessing"}},			-- Blessing of Might
			
			[SpellName(25782)]= {dur = 1800, tags = {"targetclass", "blessing"}},	-- Greater Blessing of Might
			
			[SpellName(31884)]= {dur = 20, tags = {"self"}},						-- Avenging Wrath
		}
	elseif class == "PRIEST" then -- no race abilities yet
		classTable = {
			-- discipline trainer
			[SpellName(1243)]= {dur = 1800, tags = {"target"}},			-- Power Word: Fortitude
			[SpellName(17)]= {dur = 30, tags = {"target"}},				-- Power Word: Shield
			[SpellName(6346)]= {dur = 180, tags = {"target"}},			-- Fear Ward
			[SpellName(14752)]= {dur = 1800, tags = {"target"}},		-- Divine Spirit
			[SpellName(10060)]= {dur = 15, tags = {"target"}},			-- Power Infusion
			[SpellName(33206)]= {dur = 8, tags = {"target"}},			-- Pain Suppression
			
			[SpellName(588)]= {dur = 600, tags = {"self"}},				-- Inner Fire
			[SpellName(14751)]= {dur = -1, tags = {"self"}},			-- Inner Focus
			[SpellName(1706)]= {dur = 120, tags = {"self"}},			-- Levitate
			
			[SpellName(21562)]= {dur = 3600, tags = {"targetparty"}},	-- Prayer of Fortitude
			[SpellName(27681)]= {dur = 3600, tags = {"targetparty"}},	-- Prayer of Spirit
			-- holy trainer nothing apparently
			-- shadow trainer
			[SpellName(586)]= {dur = 10, tags = {"self"}},				-- Fade
			[SpellName(2096)]= {dur = 60, tags = {"self"}},				-- Mind Vision self or target unsure...
			[SpellName(605)]= {dur = 60, tags = {"self"}},				-- Mind Control self or target unsure...
			[SpellName(15473)]= {dur = -1, tags = {"self", "stance"}},	-- Shadowform
			[SpellName(34433)]= {dur = 15, tags = {"self"}},			-- Shadowfiend well see...
			[SpellName(976)]= {dur = 600, tags = {"target"}},			-- Shadow Protection
			[SpellName(27683)]= {dur = 1200, tags = {"targetparty"}},	-- Prayer of Shadow Protection
			}
	elseif class == "ROGUE" then 
		classTable = {
		}
	elseif class == "SHAMAN" then 
		--toDo: use integers, even more use multipliers, but I dunno if this is worth the trouble since the functions arent called that often) (self=1, party=2 and so on)
		classTable = {
			[SpellName(24398)] = {dur = 600, count = 3, tags = {"self", "shield"}},	-- watershield
			[SpellName(324)]  = {dur = 600, count = 3, tags = {"self", "shield"}}, -- lightning shield
			[SpellName(16261)] = {dur = 600, tags = {"self", "shield"}},	-- improved lightning shield; unsure if needed
			[SpellName(974)]  = {dur = 600, count = 6, tags = {"target", "shield"}},	-- Earth Shield
			[SpellName(30823)] = {dur = 15, tags = {"self"}},				-- Shamanistic Rage
			[SpellName(16166)] = {dur = -1, tags = {"self"}},				-- Elemental Mastery
			[SpellName(16188)] = {dur = -1, tags = {"self"}},				-- Nature's Swiftness
			
			[SpellName(131)] = {dur = 600, tags = {"target"}},				-- water breathing
			[SpellName(546)] = {dur = 600, tags = {"target"}},				-- water walking
			
			[SpellName(8017)] = {dur = 1800, tags = {"self", "weapon"}},	-- Rockbiter Weapon
			[SpellName(8024)] = {dur = 1800, tags = {"self", "weapon"}},	-- Flametongue Weapon
			[SpellName(8033)] = {dur = 1800, tags = {"self", "weapon"}},	-- Frostbrand Weapon
			[SpellName(8232)] = {dur = 1800, tags = {"self", "weapon"}},	-- Windfury Weapon

			[SpellName(2825)] = {dur = 40, tags = {"party"}},				-- Bloodlust
			[SpellName(32182)] = {dur = 40, tags = {"party"}},				-- Heroism
			
			-- ghost wolf
			--Far Sight 6196
			--sdn
		}
	elseif class == "WARRIOR" then 
		classTable = {
			[SpellName(2565)] = {dur = 5, tags = {"self"}},		-- shield block duration or until first blocked...
			[SpellName(12945)] = {dur = 6, tags = {"self"}},	-- improved shield block +1 second damn you
			[SpellName(871)] = {dur = 10, tags = {"self"}},		-- shield wall
			[SpellName(12312)] = {dur = 13, tags = {"self"}},	-- improved shield wall +3 cries
--			[SpellName(29598)] = {tags = {"self"}},	-- shield mastery
		}
	elseif class == "WARLOCK" then
		classTable = {

		}
	end
	return classTable
end

local function GetRaceTable(race)
	if race == "troll" then
		return SpellName(20554)
	end
end

----------------------
-- Lib Functions --
----------------------
function lib:GetBuffTable(class)
	if not class then class = select(2, UnitClass("player")) end
	return GetClassTable(class)
end

function lib:GetBuffSpells(arg1, arg2, arg3, arg4, arg5)
	local classBuffTable = GetClassTable(select(2, UnitClass("player")))
	local buffTable = {}
	local args={arg1,arg2,arg3,arg4,arg5}

	for name in pairs(classBuffTable) do
		-- check if the player has actually learned the spell
		--GetSpellInfo returns nil if spell is not in SpellBook and name is used instead of SpellID. quite useful :)
		if (SpellName(name)) then 
			local isArgFound ={}
			local i=1
			while true do
				local arg = args[i]
				if not arg then break end
				table_insert(isArgFound, false)
				for _, tag in pairs(classBuffTable[name].tags) do
					if arg == tag then isArgFound[i] = true end
				end
				i=i+1				
			end
			local areAllArgsFound = true
			if (isArgFound ~= nil) then
				for _, aBoolean in ipairs(isArgFound) do
					if aBoolean == nil then break end
					if not aBoolean then areAllArgsFound = false end
				end
			end
			if areAllArgsFound then
--				table_insert(buffTable,name)
				buffTable[name] = {}
				buffTable[name].dur = classBuffTable[name].dur
				if classBuffTable[name].count then buffTable[name].count = classBuffTable[name].count end
--				DEFAULT_CHAT_FRAME:AddMessage("SelfBuffSpell: "..name)
			end
		end
	end		
	return buffTable
end

function lib:IsSelfBuffSpell(spell, class)
	if not class then class = select(2, UnitClass("player")) end
	local classBuffTable = GetClassTable(class)
	for name in pairs(classBuffTable) do
		if name == SpellName(spell) then return true end
	end
	return false
end

function lib:GetBuffDuration(spell, class)
	if not class then class = select(2, UnitClass("player")) end
	local classBuffTable = GetClassTable(class)
	for name in pairs(classBuffTable) do
		if name == SpellName(spell) then return classBuffTable[name].dur end
	end
	return nil
end

function lib:GetBuffMaxCount(spell, class)
	if not class then class = select(2, UnitClass("player")) end
	local classBuffTable = GetClassTable(class)
	for name in pairs(classBuffTable) do
		if name == SpellName(spell) then
			return classBuffTable[name].count
		end
	end
	return nil
end

-- get buffIndex, untilCancelled from spellName
-- otherwise like GetPlayerBuff (see filters)
-- CANCELABLE - Show buffs that can cancelable
-- HARMFUL - Show harmful buffs
-- HELPFUL - Show helpful buffs
-- NOT_CANCELABLE - Show buffs that can't be cancelled
function lib:GetPlayerBuff_FromSpellName(spell, filter)
	local _, _, spellNameIcon = _GetSpellInfo(spell)
		for i=1, 40 do
			local buffIndex, untilCancelled = _GetPlayerBuff(i, filter)
			if buffIndex == 0 then break end
			if spellNameIcon == _GetPlayerBuffTexture(buffIndex) then
				return buffIndex, untilCancelled
			end
		end
	return 0, nil
end

--the following are obviously pretty unefficient if used in a row, in that case you might want to only use the above then switch to Blizzard API

-- untilCancelled is packaged as returnValue -1, buff not present as 0 (would nil be better for someone?)
function lib:GetPlayerBuffTimeLeft_FromSpellName(spell, filter)
--	DEFAULT_CHAT_FRAME:AddMessage("LibSpellBuff: spell: "..spell)
	local buffIndex, untilCancelled = self:GetPlayerBuff_FromSpellName(spell, filter)
	if buffIndex == 0 then
		return 0
	elseif untilCancelled == 1 then -- it is Aura, Aspect, Stealth
		return -1
	else
		return _GetPlayerBuffTimeLeft(buffIndex)
	end
end

-- well, why not...anyone? since the Buff name is not always the spellName...
function lib:GetPlayerBuffName_FromSpellName(spell, filter)
	local buffIndex, untilCancelled = self:GetPlayerBuff_FromSpellName(spell, filter)
	if buffIndex == 0 then
		return nil
	else
		return _GetPlayerBuffName(buffIndex)
	end
end

function lib:GetPlayerBuffApplications_FromSpellName(spell, filter)
	local buffIndex, untilCancelled = self:GetPlayerBuff_FromSpellName(spell, filter)
	if buffIndex == 0 then
		return nil
	else
		return _GetPlayerBuffApplications(buffIndex)
	end
end

function lib:GetPlayerBuffDispelType_FromSpellName(spell, filter)
	local buffIndex, untilCancelled = self:GetPlayerBuff_FromSpellName(spell, filter)
	if buffIndex == 0 then
		return nil
	else
		return _GetPlayerBuffDispelType(buffIndex)
	end
end

-- I doubt this is useful....however for the moment
function lib:GetPlayerBuffTexture_FromSpellName(spell, filter)
	local buffIndex, untilCancelled = self:GetPlayerBuff_FromSpellName(spell, filter)
	if buffIndex == 0 then
		return nil
	else
		return _GetPlayerBuffTexture(buffIndex)
	end
end

------------------------------
-- Lib Event Functions --
------------------------------
--[[
function lib:PLAYER_LOGIN()

	--self.EventFrame:RegisterEvent("LEARNED_SPELL_IN_TAB");
	--lib:LEARNED_SPELL_IN_TAB();
	--lib:GetSelfBuffSpellsGeneric()
end

function lib:PLAYER_ENTERING_WORLD()
--	lib:GetBuffSpells("self")
end
]]--