local addon = Poppins
if not addon then return end
local module = addon:NewModule("Durations", "AceEvent-3.0", "AceHook-3.0")

local LibSpellsBuff 

local _GetSpellInfo = GetSpellInfo
local _GetSpellName = GetSpellName
local _GetTime = GetTime

local Frame = addon.Frame
local Button = addon.Button
local PopButton = addon.PopButton

local playerName = UnitName("player")
local allCastableBuffs = {}
local buffMonitor = {}
local debugmode = true

local function get(key)
	local option = key[#key]
	return module.db.profile[option]
end
local options = {
	name = module.moduleName,
	desc = module.moduleName,
	type = "group",
	handler = module,
	args = {
		enable = {
			name = "Enable",
			desc = "Enables / disables the module",
			type = "toggle",
			get = "IsEnabled",
			set = function(key, value)
				if value then
					module:Enable()
				else
					module:Disable()
				end
			end,
		},
		cdreverse = {
			name = "Reverse CD",
			desc = "Dawn or Dust?",
			type = "toggle",
			get = get,
			set = function(key, value)
				local option = key[#key]
				module.db.profile[option] = value
			end,			
		},	
	},
}
local defaultDB = {
	profile = {
		enabled = false,
		cdreverse = true,
	}
}

------------------------
-- local functions --
------------------------

local function GetSpellNameFromButton(button)
	local type, value = button.type, button.value
	if type == "spell" and value then
		return _GetSpellName(value)
	end
end

local function IsCastableBuff(spellName)
	for _, name in ipairs(allCastableBuffs) do
		if spellName == name then return true end
	end
	return false
end

--------------------------
-- module functions --
--------------------------

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("Durations", defaultDB)
	self.db.RegisterCallback(module, "OnProfileChanged")
	self.db.RegisterCallback(module, "OnProfileReset", "OnProfileChanged")
	self:SetEnabledState(self.db.profile.enabled)
	
	addon.options.args.modules.args[module.moduleName] = options
end

--local selfBuffTable = {}
function module:OnEnable()
	self.db.profile.enabled = true
	--self:RegisterEvent("PLAYER_AURAS_CHANGED", "IteratePlayerBuffs")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	addon.PopButton.prototype.RegisterCallback(self, "OnPopButtonSetAction", "ScanAuras")
	self:RawHook(Button.prototype, "UpdateCooldown", nil, true)
	
	LibSpellsBuff = LibStub("LibSpellsBuff-1.0")
	if not LibSpellsBuff then DEFAULT_CHAT_FRAME:AddMessage("LibSpellsBuff-1.0 not loaded!") return
	else DEFAULT_CHAT_FRAME:AddMessage("LibSpellsBuff-1.0 loaded by "..addon.name..": "..module.moduleName) end
	allCastableBuffs = LibSpellsBuff:GetBuffSpells()
	self:ScanAuras()
--	local totemModule = addon:GetModule("TotemTimer")
--	if totemModule then DEFAULT_CHAT_FRAME:AddMessage(module.moduleName.."has access to the TotemTimer!") end
end

function module:OnDisable()
	self.db.profile.enabled = false
	self:UnregisterAllEvents()
	self:UnhookAll()
	for frame in Frame:Iterate() do
		frame.button:UpdateCooldown()
	end
	for button in PopButton:Iterate() do
		button:UpdateCooldown()
	end
end

module.OnProfileChanged = addon.OnProfileChanged

function module:ScanAuras()
	if not allCastableBuffs then return end
	for spellName in pairs(allCastableBuffs) do
		for button in PopButton:Iterate() do
			buttonSpellName = GetSpellNameFromButton(button)
			if buttonSpellName == spellName then
				if not buffMonitor[spellName] then buffMonitor[spellName] = {} end
				buffMonitor[spellName].startTime = _GetTime() - allCastableBuffs[spellName].dur + LibSpellsBuff:GetPlayerBuffTimeLeft_FromSpellName(spellName, "HELPFUL")
				buffMonitor[spellName].count = LibSpellsBuff:GetPlayerBuffApplications_FromSpellName(spellName, "HELPFUL")
				self:UpdateCooldown(button) 
			end
		end
		for frame in Frame:Iterate() do
			buttonSpellName = GetSpellNameFromButton(frame.button)
			if buttonSpellName == spellName then
				self:UpdateCooldown(frame.button)
			end
		end
	end
end

-- iterate *currently* through all castable buffs on self
function module:IteratePlayerBuffs()
	if not allCastableBuffs then return end
	for spellName in pairs(allCastableBuffs) do
		self:UpdateDuration(spellName)
	end
end

function module:UpdateDuration(spellName)
	local buttonSpellName
	for frame in Frame:Iterate() do
		buttonSpellName = GetSpellNameFromButton(frame.button)
		if buttonSpellName == spellName then
--			DEFAULT_CHAT_FRAME:AddMessage("Updating Button: "..spellName)
			--self:UpdateCooldown(frame.button)
			frame.button:UpdateCooldown()
		end
	end
	for button in PopButton:Iterate() do
		buttonSpellName = GetSpellNameFromButton(button)
		if buttonSpellName == spellName then
--			DEFAULT_CHAT_FRAME:AddMessage("Updating PopButton: "..spellName)
			--self:UpdateCooldown(button) 
			button:UpdateCooldown()
		end
	end
end

function module:UpdateCooldown(button)
	local db = self.db.profile
	local cooldown = button.cooldown
	local spellName = GetSpellNameFromButton(button)

	if spellName then
		if allCastableBuffs[spellName] then
			local startTime = buffMonitor[spellName].startTime
			if not startTime then return end
			
			local spellDuration = allCastableBuffs[spellName].dur
			if type(spellDuration) == "table" then
				spellDuration = spellDuration[1] --this is just a reminder for now
			end	
			if not spellDuration then return end
--			DEFAULT_CHAT_FRAME:AddMessage(spellName.."s Duration: "..spellDuration)
			
			local spellTimeLeft = startTime + spellDuration - _GetTime()
--			DEFAULT_CHAT_FRAME:AddMessage(spellName.."s spellTimeLeft: "..spellTimeLeft)
			
			if spellTimeLeft > 0 then
				cooldown:SetReverse(db.cdreverse)
				cooldown:SetCooldown(startTime, spellTimeLeft)
				cooldown.reversed = db.cdreverse
				local count = buffMonitor[spellName].count
				if count then button.count:SetText(count) end
				return
			else
				button:UpdateCount()
			end
		end
	end
	if cooldown.reversed then
		cooldown:SetReverse(false)
		cooldown.reversed = nil
	end	
	self.hooks[Button.prototype].UpdateCooldown(button)
end

-- Event Handler
function module:COMBAT_LOG_EVENT_UNFILTERED(_, timestamp, event, _, srcName, srcFlags, _, dstName, dstFlags, _, spellName)
-- player: srcFlags == 1297	
	if buffMonitor[spellName] then
		if srcFlags == 1297 and event == "SPELL_CAST_SUCCESS" then			
			if dstFlags == 1297 or dstFlags == -2147483648 or spellName == _GetSpellInfo(32594) then --ES
--				DEFAULT_CHAT_FRAME:AddMessage("Casted: "..spellName.." dstFlags: "..dstFlags)

				buffMonitor[spellName].timestamp = timestamp
				buffMonitor[spellName].startTime = _GetTime()
				
				if allCastableBuffs[spellName].count then
					buffMonitor[spellName].count = allCastableBuffs[spellName].count
				end

				if spellName == _GetSpellInfo(32594) then
					buffMonitor.dstName = dstName
--					DEFAULT_CHAT_FRAME:AddMessage("Monitored dstName: "..dstName)
				end
				self:UpdateDuration(spellName)
			end
		elseif (dstFlags == 1297 or (buffMonitor.dstName and buffMonitor.dstName == dstName)) and buffMonitor[spellName] then
			if event == "SPELL_HEAL" or event == "SPELL_ENERGIZE" then
				buffMonitor[spellName].count = buffMonitor[spellName].count - 1
			elseif event == "SPELL_DISPEL" or event == "SPELL_STOLEN" then 
				buffMonitor[spellName].startTime = nil
			elseif event == "SPELL_AURA_REMOVED" and _GetTime() - buffMonitor[spellName].startTime > 0.5 then
				buffMonitor[spellName].startTime = nil
			elseif event == "SPELL_CAST_SUCCESS" or event == "SPELL_AURA_REFRESHED" then
				-- overwritten by someone else (hmm, inspect the new SPELL_AURA_REFRESHED thingie)
				if spellName == _GetSpellInfo(32594) then
					buffMonitor[spellName].startTime = nil
				else
					buffMonitor[spellName].startTime = _GetTime()
					if event == "SPELL_AURA_REFRESHED" then
						DEFAULT_CHAT_FRAME:AddMessage("SPELL_AURA_REFRESHED: "..spellName)
					end
				end
			end
			if buffMonitor[spellName].count and not buffMonitor[spellName].startTime then
				buffMonitor[spellName].count = nil
			end
			self:UpdateDuration(spellName)
		end
	elseif event == "UNIT_DIED" then
		if dstFlags == 1297 then
			for spellName in pairs(buffMonitor) do
				buffMonitor[spellName].startTime = nil
				buffMonitor[spellName].count = nil
			end
			self:IteratePlayerBuffs()
		elseif buffMonitor.dstName and buffMonitor.dstName == dstName then
			spellName = _GetSpellInfo(32594)
			buffMonitor[spellName].startTime = nil
			buffMonitor[spellName].count = nil
			self:UpdateDuration(spellName)
		end
	end
end
