if not select(2, UnitClass("player")) == "SHAMAN" then return end

local addon = Poppins
if not addon then return end
local module = addon:NewModule("TotemTimer", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

local GetSpellName = GetSpellName
local GetTotemInfo = GetTotemInfo

local Frame = addon.Frame
local Button = addon.Button
local PopButton = addon.PopButton

local slotSpells = {}
local slotTimer = {}
local totemSlots = {
	-- Earth
	[GetSpellInfo(8143)]	= 2,	-- Tremor Totem
	[GetSpellInfo(5730)]	= 2,	-- Stoneclaw Totem
	[GetSpellInfo(2484)]	= 2,	-- Earthbind Totem
	[GetSpellInfo(8075)]	= 2,	-- Strength of Earth Totem
	[GetSpellInfo(8071)]	= 2,	-- Stoneskin Totem
	[GetSpellInfo(2062)]	= 2,	-- Earth Elemental Totem
	-- Fire
	[GetSpellInfo(30706)]	= 1,	-- Totem of Wrath
	[GetSpellInfo(3599)]	= 1,	-- Searing Totem
	[GetSpellInfo(8190)]	= 1,	-- Magma Totem
	[GetSpellInfo(1535)]	= 1,	-- Fire Nova Totem
	[GetSpellInfo(8227)]	= 1,	-- Flametongue Totem
	[GetSpellInfo(8181)]	= 1,	-- Frost Resistance Totem
	[GetSpellInfo(2894)]	= 1,	-- Fire Elemental Totem
	-- Water
	[GetSpellInfo(5675)]	= 3,	-- Mana Spring Totem
	[GetSpellInfo(5394)]	= 3,	-- Healing Stream Totem
	[GetSpellInfo(8166)]	= 3,	-- Poison Cleansing Totem
	[GetSpellInfo(8170)]	= 3,	-- Disease Cleansing Totem
	[GetSpellInfo(16190)]	= 3,	-- Mana Tide Totem
	[GetSpellInfo(8184)]	= 3,	-- Fire Resistance Totem
	-- Air
	[GetSpellInfo(3738)]	= 4,	-- Wrath of Air Totem
	[GetSpellInfo(8177)]	= 4,	-- Grounding Totem
	[GetSpellInfo(8835)]	= 4,	-- Grace of Air Totem
	[GetSpellInfo(8512)]	= 4,	-- Windfury Totem
	[GetSpellInfo(15107)]	= 4,	-- Windwall Totem
	[GetSpellInfo(10595)]	= 4,	-- Nature Resistance Totem
	[GetSpellInfo(25908)]	= 4,	-- Tranquil Air Totem
	[GetSpellInfo(6495)]	= 4,	-- Sentry Totem
}
local totemPulses = {
	-- EARTH
	[GetSpellInfo(8143)]	= 3,	-- Tremor Totem
	[GetSpellInfo(5730)]	= 2,	-- Stoneclaw Totem
	[GetSpellInfo(2484)]	= 3,	-- Earthbind Totem
	-- FIRE
	[GetSpellInfo(8190)]	= 2,	-- Magma Totem
	[GetSpellInfo(8227)]	= 10,	-- Flametongue Totem
	-- WATER
	[GetSpellInfo(5675)]	= 2,	-- Mana Spring Totem
	[GetSpellInfo(5394)]	= 2,	-- Healing Stream Totem
	[GetSpellInfo(8166)]	= 5,	-- Poison Cleansing Totem
	[GetSpellInfo(8170)]	= 5,	-- Disease Cleansing Totem
	[GetSpellInfo(16190)]	= 3,	-- Mana Tide Totem
	-- AIR
	[GetSpellInfo(8512)]	= 10,	-- Windfury Totem
}
local function get(key)
	local option = key[#key]
	return module.db.profile[option]
end
local options = {
	name = module.moduleName,
	desc = module.moduleName,
	type = "group",
	handler = module,
	args = {
		enable = {
			name = "Enable",
			desc = "Enables / disables the module",
			type = "toggle",
			get = "IsEnabled",
			set = function(key, value)
				if value then
					module:Enable()
				else
					module:Disable()
				end
			end,
		},
		timer = {
			name = "Enable Timer",
			desc = "Enables / disables the Totem Timer",
			type = "toggle",
			get = get,
			set = function(key, value)
				local option = key[#key]
				module.db.profile[option] = value
				for frame in Frame:Iterate() do
					frame.button:UpdateCooldown()
				end
				for button in PopButton:Iterate() do
					if button:IsVisible() then
						button:UpdateCooldown()
					end
				end
			end,
		},
		sharedTimer = {
			name = "Shared Timer",
			desc = "Totems of the same element share the same timer",
			type = "toggle",
			get = get,
			set = function(key, value)
				local option = key[#key]
				module.db.profile[option] = value
				for frame in Frame:Iterate() do
					frame.button:UpdateCooldown()
				end
				for button in PopButton:Iterate() do
					if button:IsVisible() then
						button:UpdateCooldown()
					end
				end
			end,
		},
		pulser = {
			name = "Enable Pulser",
			desc = "Enables / disables the Totem Pulser",
			type = "toggle",
			get = get,
			set = function(key, value)
				local option = key[#key]
				module.db.profile[option] = value
			end,
		},
		cdreverse = {
			name = "Reverse CD",
			desc = "Light to dark or dark to light?",
			type = "toggle",
			get = get,
			set = function(key, value)
				local option = key[#key]
				module.db.profile[option] = value
			end,			
		},	
	},
}
local defaultDB = {
	profile = {
		enabled = true,
		timer = true,
		pulser = true,
		sharedTimer = false,
		cdreverse = true,
	}
}

local function hasTotem(button)
	local type, value = button.type, button.value
	if type == "spell" and value then
		local spellName = GetSpellName(value)
		if spellName and totemSlots[spellName] then
			return spellName
		end
	end
end
local function showPulse(spellName)
	if not spellName or not module.db.profile.pulser then return end
	for frame in Frame:Iterate() do
		local button = frame.button
		if hasTotem(button) == spellName then
			module:ShowPulse(button, spellName)
		end
	end
	for button in PopButton:Iterate() do
		if button:IsVisible() and hasTotem(button) == spellName then
			module:ShowPulse(button, spellName)
		end
	end
end

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("TotemTimer", defaultDB)
	self.db.RegisterCallback(module, "OnProfileChanged")
	self.db.RegisterCallback(module, "OnProfileReset", "OnProfileChanged")
	self:SetEnabledState(self.db.profile.enabled)

	addon.options.args.modules.args[module.moduleName] = options
end

function module:OnEnable()
	self.db.profile.enabled = true
	self:RegisterEvent("PLAYER_TOTEM_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	--part of the non-working right click overwrite
--[[
	addon.PopButton.prototype.RegisterCallback(self, "OnPopButtonSetAction")
	--were missing the first SetAllActions hmmmmmmmsuckz
	for popbutton in PopButton:Iterate() do
		for modifier, attributes in pairs(popbutton.db) do
			for action, value in pairs(attributes) do
				self:OnPopButtonSetAction(popbutton, action, modifier, value)
			end
		end
	end
]]--
	self:RawHook(Button.prototype, "UpdateCooldown", nil, true)
end

function module:OnDisable()
	self.db.profile.enabled = false
	self:UnregisterAllEvents()

	self:CancelAllTimers()
	for slot, timer in pairs(slotTimer) do
		slotTimer[slot] = nil
	end

	self:UnhookAll()
	for frame in Frame:Iterate() do
		frame.button:UpdateCooldown()
	end
	for button in PopButton:Iterate() do
		if button:IsVisible() then
			button:UpdateCooldown()
		end
	end
end

module.OnProfileChanged = addon.OnProfileChanged

function module:UpdateCooldown(button)
	local db = self.db.profile
	local cooldown = button.cooldown
	if db.timer then
		local totemName = hasTotem(button)
		local slot = totemSlots[totemName]
		if totemName and ((not button.id and db.sharedTimer and slotSpells[slot]) or totemName == slotSpells[slot]) then
			local _, _, startTime, duration = GetTotemInfo(slot)
			cooldown:SetReverse(db.cdreverse)
			cooldown:SetCooldown(startTime, duration)
			cooldown.reversed = db.cdreverse
			return
		end
	end
	if cooldown.reversed then
		cooldown:SetReverse(false)
		cooldown.reversed = nil
	end
	self.hooks[Button.prototype].UpdateCooldown(button)
end

function module:UNIT_SPELLCAST_SUCCEEDED(event, unit, spellName)
	if (unit ~= "player") then return end
	local slot = totemSlots[spellName]
	if not slot then return end
	slotSpells[slot] = spellName

	local pulse = totemPulses[spellName]
	if pulse then
		local timer = slotTimer[slot]
		if timer then
			self:CancelTimer(timer)
		end
		slotTimer[slot] = module:ScheduleRepeatingTimer(showPulse, pulse, spellName)
	end
end

function module:PLAYER_TOTEM_UPDATE(event, slot)
	local _, totemName = GetTotemInfo(slot)
	if slotSpells[slot] then
		if totemName == "" then
			slotSpells[slot] = nil
		end
		if self.db.profile.timer then
			for frame in Frame:Iterate() do
				frame.button:UpdateCooldown()
			end
			for button in PopButton:Iterate() do
				if button:IsVisible() then
					button:UpdateCooldown()
				end
			end
		end
	end

	local timer = slotTimer[slot]
	if totemName == "" and timer then
		self:CancelTimer(timer)
		slotTimer[slot] = nil
	end
end

function module:ShowPulse(button, spellName)
	if not button.pulse then
		button.pulse = CreateFrame("Cooldown", button:GetName().."Pulse", button, "CooldownFrameTemplate")
		--button.pulse:SetReverse(true)
	end
	--button.pulse:SetCooldown(GetTime(), totemPulses[spellName])
	button.pulse:SetCooldown(0, 0)
end

--overwrite Attributes if Totem; doesnt work one bit atm :(
function module:OnPopButtonSetAction(popbutton, action, modifier, value)
	--DEFAULT_CHAT_FRAME:AddMessage("TotemTimer got Callback OnPopButtonSetAction")
	local slot = totemSlots[value]
	if slot then
--		DEFAULT_CHAT_FRAME:AddMessage("slot: "..slot)
--		DEFAULT_CHAT_FRAME:AddMessage("action: "..action)
--		DEFAULT_CHAT_FRAME:AddMessage("modifier: "..modifier)
--		DEFAULT_CHAT_FRAME:AddMessage("value: "..value)
		modifier = "*"
		action = "macrotext"
		value = "/run DestroyTotem("..slot..")"
		id = popbutton.id
		--SetAttribute(("%s%s-%d"):format(modifier, action, id), value)
--		popbutton.parent.button:SetAttribute(("%s%s2-%d"):format(modifier, action, id), value)
--		popbutton.parent.button:SetAttribute(("%s%s2-%d"):format(modifier, action, id), value)
--		popbutton.parent.button:SetAttribute("*type2", "macro")
--		popbutton.parent.button:SetAttribute("macrotext", "/run DestroyTotem("..slot..")")
	end
end
