local addon = Poppins
if not addon then return end

local LibMacroParser = LibStub("LibMacroParser-1.0")
local LibButtonFacade = LibStub("LibButtonFacade", true)
local LibStickyFrames = LibStub("LibStickyFrames-2.0")

local count = 0
local weak = {__mode='k'}
local cache = setmetatable({}, weak)
local used = setmetatable({}, weak)

local PopButton = setmetatable(CreateFrame("CheckButton"), addon.Button.metatable)
LibStub("AceEvent-3.0"):Embed(PopButton)
PopButton.callbacks = LibStub("CallbackHandler-1.0"):New(PopButton)
addon.PopButton = {}
addon.PopButton.prototype = PopButton
addon.PopButton.metatable = {__index = PopButton}

function addon.PopButton:Retrieve()
	local button = next(cache)
	if button then
		cache[button] = nil
	else
		count = count + 1
		local rid = count
		local header = addon.header
		local buttonName = format("%sPopButton%d", addon.name, rid)
		button = setmetatable(CreateFrame("CheckButton", buttonName, header, "SecureAnchorUpDownTemplate,SecureActionButtonTemplate,ActionButtonTemplate"), addon.PopButton.metatable)
		button.state = ""
		button.rid = rid
		button:SetWidth(36)
		button:SetHeight(36)

		button.icon = _G[("%sIcon"):format(buttonName)]
		button.border = _G[("%sBorder"):format(buttonName)]
		button.cooldown = _G[("%sCooldown"):format(buttonName)]
		button.macroName = _G[("%sName"):format(buttonName)]
		button.hotkey = _G[("%sHotKey"):format(buttonName)]
		button.count = _G[("%sCount"):format(buttonName)]
		button.flash = _G[("%sFlash"):format(buttonName)]
		button.flash:Hide()

		header:SetAttribute("addchild", button)

		button.actions = {}
		button.attributes = {}
		button:HookScript("OnEnter", button.ShowTooltip)
		button:HookScript("OnLeave", button.HideTooltip)
		button:SetScript("OnAttributeChanged", button.OnAttributeChanged)

		button:RegisterForDrag("LeftButton", "RightButton")
	
		button.callbacks:Fire("OnPopButtonCreate", self)
	end
	used[button] = true
	return button
end

function addon.PopButton:Iterate()
	return pairs(used)
end

function PopButton:Setup(id, parent, db)
	id, parent, db = id or self.id, parent or self.parent, db or self.db
	self:Reset()
	if not id or not parent or not db then return end

	self.db = db
	self.id = id
	self.parent = parent

	LibButtonFacade = LibButtonFacade or LibStub("LibButtonFacade", true)
	if LibButtonFacade then
		LibButtonFacade:Group(addon.name, self.parent.db.group or tostring(parent.id)):AddButton(self)
	end

	LibStickyFrames:RegisterFrame(self)
	LibStickyFrames:SetFrameInsets(self, -3, -2, -2, -3)
	LibStickyFrames:SetFrameGroup(self, parent)
	LibStickyFrames:SetFrameText(self, ("%d-%d"):format(parent.id, id))

	local rid = self.rid
	local header = addon.header

	self:RegisterAllEvents()
	self:SetScript("OnShow", self.OnShow)
	self:SetScript("OnHide", self.OnShow)
	self:SetScript("OnDragStart", self.OnDragStart)
	self:SetScript("OnReceiveDrag", self.OnReceiveDrag)
	self:SetScript("PreClick", self.PreClick)
	self:SetScript("PostClick", self.PostClick)
	self:SetScript("OnEnter", self.ShowTooltip)
	self:SetScript("OnLeave", self.HideTooltip)

--	local addonDB = addon.db.profile
	local parentDB = parent.db	

	local clicks
	if parentDB.left == 1 then
		clicks = clicks and strjoin(",", clicks, "LeftButtonUp") or "LeftButtonUp"
		self:SetAttribute("onmouseupbutton1", "mub1")
		self:SetAttribute("*childstate-mub1", id)
	elseif parentDB.left == 2 then
		clicks = clicks and strjoin(",", clicks, "LeftButtonUp") or "LeftButtonUp"
	elseif parentDB.left == 3 then
		self:SetAttribute("onmouseupbutton1", "mub1")
		self:SetAttribute("*childstate-mub1", id)
	end
	if parentDB.right == 1 then
		clicks = clicks and strjoin(",", clicks, "RightButtonUp") or "RightButtonUp"
		self:SetAttribute("onmouseupbutton2", "mub2")
		self:SetAttribute("*childstate-mub2", id)
	elseif parentDB.right == 2 then
		clicks = clicks and strjoin(",", clicks, "RightButtonUp") or "RightButtonUp"
	elseif parentDB.right == 3 then
		self:SetAttribute("onmouseupbutton2", "mub2")
		self:SetAttribute("*childstate-mub2", id)
	end
	if parentDB.middle == 1 then
		clicks = clicks and strjoin(",", clicks, "MiddleButtonUp") or "MiddleButtonUp"
		self:SetAttribute("onmouseupbutton3", "mub3")
		self:SetAttribute("*childstate-mub3", id)
	elseif parentDB.middle == 2 then
		clicks = clicks and strjoin(",", clicks, "MiddleButtonUp") or "MiddleButtonUp"
	elseif parentDB.middle == 3 then
		self:SetAttribute("onmouseupbutton3", "mub3")
		self:SetAttribute("*childstate-mub3", id)
	end
	if clicks then
		self:RegisterForClicks(strsplit(",", clicks))
	end

	self:SetAttribute("anchorchild", parent)
	self:SetAttribute("showstates", ("-1,%d"):format(parent.id))
	parent:SetAttribute(("statemap-anchor-%d"):format(id), id)

--	local parentDB = parent.db
	self:SetScale(parentDB.popscale)
	self:Anchor(parent, parentDB.poppoint, parentDB.padding)
	--PopButton FrameStrata
	self:SetFrameStrata(parentDB.popframestrata)

	local state = addon.header:GetAttribute("state")
	if state == "-1" or state == tostring(parent.id) then
		if self:IsShown() then
			LibStickyFrames:SetFrameHidden(self)
		else
			self:Show()
		end
	else
		self:Hide()
	end
	self:SetLock(addon:IsLocked())

	self:SetAllActions()
	self.callbacks:Fire("OnPopButtonSetup", self, id, parent, db)
end

function PopButton:Reset()
	self.callbacks:Fire("OnPopButtonReset", self)
	local parent = self.parent

	LibButtonFacade = LibButtonFacade or LibStub("LibButtonFacade", true)
	if LibButtonFacade and parent then
		LibButtonFacade:Group(addon.name, tostring(parent.id)):RemoveButton(self)
	end

	LibStickyFrames:UnregisterFrame(self)
	LibMacroParser:UnregisterAllMacros(self)
	self:UnregisterAllEvents()
	self:SetScript("OnShow", nil)
	self:SetScript("OnHide", nil)
	self:SetScript("OnDragStart", nil)
	self:SetScript("OnReceiveDrag", nil)
	self:SetScript("OnUpdate", nil)
	self:SetScript("PreClick", nil)
	self:SetScript("PostClick", nil)
	self:SetScript("OnEnter", nil)
	self:SetScript("OnLeave", nil)

	for attribute, value in pairs(self.actions) do
		self:SetAction(attribute, "", nil)
	end
	for name, value in pairs(self.attributes) do
		self:SetAttribute(name, nil)
	end

	self:ClearAllPoints()
	self:Hide()

	self.db = nil
	self.id = nil
	self.parent = nil
end

function PopButton:Store()
	used[self] = nil

	self:Reset()

	cache[self] = true
end

function PopButton:OnDragStart()
	if InCombatLockdown() or addon.db.profile.lock then return end

	local db = self.db
	local modifier = SecureButton_GetModifierPrefix()

	self:Pickup(modifier)

	db[modifier] = nil
	self:SetAllActions()
end

function PopButton:GetCursorInfo()
	local type, value, value2 = GetCursorInfo()
	if type == "spell" then
		local spell, rank, maxRank
		spell, rank = GetSpellName(value, value2)
		spell, maxRank = GetSpellName(spell)
		if rank == maxRank then
			value = spell
		else
			value = ("%s(%s)"):format(spell, rank or "")
		end
	elseif type == "item" then
		value = "item:"..value
	end
	return type, value
end

function PopButton:OnReceiveDrag()
	if InCombatLockdown() or not (CursorHasItem() or CursorHasSpell() or CursorHasMacro()) then return end

	local type, value = self:GetCursorInfo()

	ClearCursor()
	local modifier = SecureButton_GetModifierPrefix()
	self:Pickup(modifier)

	local db = self.db
	db[modifier].type = type
	db[modifier][type] = value

	self:SetAllActions()
end

function PopButton:Pickup(modifier)
	if not modifier then modifier = "" end
	local type = self:GetAction("type", modifier)
	if not type then return end
	local value = self:GetAction(type, modifier)
	if not value then
		return
	elseif type == "spell" then
		PickupSpell(value)
	elseif type == "item" then
		PickupItem(value)
	elseif type == "macro" then
		PickupMacro(value)
	end
end

local pickupType, pickupValue
function PopButton:PreClick()
	self:OnReceiveDrag()
	pickupType, pickupValue = self:GetCursorInfo()
end

function PopButton:PostClick()
	if not InCombatLockdown() and pickupType and pickupValue then
		if pickupType == "spell" then
			PickupSpell(pickupValue)
		elseif pickupType == "item" then
			PickupItem(pickupValue)
		elseif pickupType == "macro" then
			PickupMacro(pickupValue)
		end
	end
	pickupType, pickupValue = nil, nil
	self:UpdateState()
end

function PopButton:SetLock(v)
	self:EnableMouse(v)

	self.callbacks:Fire("OnPopButtonSetLock", self, v)
end

local relPoints = { top = "bottom", bottom = "top", left = "right", right = "left", center = "center" }
function PopButton:Anchor(frame, point, padding)
	local id = self.id - 1
	local parent = self.parent
	local db = parent.db
	local x, y = 0, 0
	self:SetFrameLevel(parent:GetFrameLevel() - 1)
	if point == "center" then
		local count = db.count
		local offset = math.fmod(db.offset or 0, 360)

		local angle = math.rad(((360 / count) * id) + offset)
		x = math.sin(angle) * (padding + self:GetWidth())
		y = math.cos(angle) * (padding + self:GetHeight())
--[[	
		self:ClearAllPoints()
		self:SetPoint(point, frame, point,  x, y)
		self:SetFramePoint(self)
]]--
		--using this instead does not throw the SetFramePoint error (forum), *but* it seems you have to create one more popbutton than there is visible, for example you need 3 popbuttons to get popbuttons 1 and two shown.
		--no idea if this fix introduces this behaviour or if it was just masked by the former error, please have a look
		LibStickyFrames:SetFramePoints(self, relPoints[point], parent.popbuttons[id] or parent, point, x, y)
	else
		if point == "top" or point == "bottom" then
			y = padding
		else
			x = padding
		end
		if point == "bottom" or point == "left" then
			x = x * -1
			y = y * -1
		end
		LibStickyFrames:SetFramePoints(self, relPoints[point], parent.popbuttons[id] or parent, point, x, y)
	end
end

function PopButton:IsSelectedState()
	local frame = self.parent
	return frame:GetAttribute("state") == tostring(self.id)
end

local function ReorderModifiers(modifier)
	local alt = modifier:match("(alt%-)") or ""
	local ctrl = modifier:match("(ctrl%-)") or ""
	local shift = modifier:match("(shift%-)") or ""
	return ("%s%s%s"):format(alt, ctrl, shift)
end

function PopButton:GetAction(action, modifier)
	local id = self.id
	if not modifier or modifier == "" then modifier = "*" end
	return self:GetAttribute(("%s%s*"):format(modifier, action))
end

function PopButton:SetAction(action, modifier, value)
	modifier = modifier or ""

	local parent = self.parent
	local id = self.id

	self.actions[("%s%s"):format(modifier, action)] = value
	if modifier == "" then modifier = "*" end
	self:SetAttribute(("%s%s*"):format(modifier, action), value)
	parent.button:SetAttribute(("%s%s-%d"):format(modifier, action, id), value)
	self.callbacks:Fire("OnPopButtonSetAction", self, action, modifier, value)
end

function PopButton:SetAllActions()
	local db = self.db
	local id = self.id

	LibMacroParser:UnregisterAllMacros(self)
	for attribute, value in pairs(self.actions) do
		self:SetAction(attribute, nil)
	end

	local selfcast = GetModifiedClick("SELFCAST")
	selfcast = selfcast and ("%s-"):format(selfcast:lower())
	for modifier, attributes in pairs(db) do
		for action, value in pairs(attributes) do
			self:SetAction(action, modifier, value)
		end
		if selfcast then
			local modifier = ReorderModifiers(("%s%s"):format(selfcast, modifier))
			if not db[modifier] then
				for action, value in pairs(attributes) do
					self:SetAction(action, modifier, value)
				end
			end
			self:SetAction("unit", modifier, "player")
		end
	end

	self:Update()
	if self:IsSelectedState() then
		self.parent.button:Update()
	end
end