Poppins = LibStub("AceAddon-3.0"):NewAddon("Poppins", "AceConsole-3.0", "AceEvent-3.0")
local addon = Poppins

local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigCmd = LibStub("AceConfigCmd-3.0")

local LibButtonFacade
local LibStickyFrames = LibStub("LibStickyFrames-2.0")
local LibKeyBound = LibStub("LibKeyBound-1.0")
local media = LibStub("LibSharedMedia-3.0")

--media setup
local fonts = {}
for k, v in pairs(media:List("font")) do
   fonts[v] = v
end
media:RegisterCallback("LibSharedMedia_Registered",
	function(event, mediatype, key)
	if mediatype == 'font' then
		fonts[key] = key
		end
	end)

local locked = true
local wasLocked = true
local function get(key)
	return addon.db.profile[key[#key]]
end
local function set(key, value)
	addon.db.profile[key[#key]] = value
	addon:UpdateButtons()
end
local buttonValues = {
	[0] = "Nothing",
	"Cast and Set",
	"Cast",
	"Set",
}

local options = {
	type = "group",
	handler = addon,
	plugins = {},
	args = {
		enable = {
			name = "Enable",
			desc = "Enables / disables the addon",
			type = "toggle",
			order = 1,
			set = function(key, value)
				if value then
					addon:Enable()
				else
					addon:Disable()
				end
			end,
			get = "IsEnabled",
		},
		lock = {
			name = "Lock",
			desc = "Locks / unlocks all the buttons",
			type = "toggle",
			order = 2,
			set = "SetLock",
			get = "IsLocked",
		},
		buttonLock = {
			name = "Button Lock",
			desc = "Prevents dragging the spells/items/macros from the popbuttons",
			type = "toggle",
			order = 3,
			set = function(key, value)
				addon.db.profile.lock = value
				for frame in addon.Frame:Iterate() do
					frame:UpdateShowPopButton()
				end
			end,
			get = function()
				return addon.db.profile.lock
			end,
		},
		assignBindings = {
			name = "Key Bindings",
			desc = "Assign Bindings for Buttons.",
			type = "toggle",
			order = 4,
			get = function() return Poppins.assignBindings end,
			set = LibKeyBound.Toggle,
			disabled = getCombatLockdown,
		},
		global ={
			name = "Global",
			desc = "Global Options",
			type = "group",
			order = 1,
			childGroups = "tab",
			plugins = {},
			args = {
				showTooltips = {
					name = "Button Tooltip",
					desc = "Configure the Button Tooltip",
					type = "select",
					get = get,
					set = set,
					values = {
						[1] = "Disabled",
						[2] = "Enabled",
						[3] = "Disabled in Combat",
					},
				},
			},
		},

		buttons = {
			name = "Buttons",
			desc = "Buttons",
			type = "group",
			order = 6,
			args = {
				new = {
					name = "New Button",
					desc = "Creates a new button",
					type = "execute",
					func = "AddButton",
					order = 1,
				},
				remove = {
					name = "Remove Button",
					desc = "Removes a button",
					type = "select",
					set = "RemoveButton",
					values = {},
					disabled = function()
						return addon.db.profile.count == 0
					end,
					order = 2,
				},
			},
		},
		groups ={
			name = "Groups",
			desc = "Groups",
			type = "group",
			order = 7,
			args = {
				new = {
					name = "New Group",
					desc = "Creates a new group",
					type = "input",
					set = "AddGroup",
				},
				remove = {
					name = "Remove group",
					desc = "Removes a group",
					type = "select",
					set = "RemoveGroup",
					values = {},
					disabled = function()
						return not next(addon.db.profile.groups)
					end,
				},
			},
		},
		modules = {
			name = "Modules",
			desc = "Modules",
			type = "group",
			order = 8,
			args = {},
			plugins = {},
		}
	},
}
local function getOptions(...)
	local t = options.args
	local n = select("#", ...)
	for i=1, n do
		t = t[(select(i, ...))].args
	end
	return t
end
local defaultDB = {
	profile = {
		enabled = true,
		count = 0,
		lock = false,
		showTooltips = 3,
		["*"] = {
			-- those appear only on buttons
			state = "1",
			group = false,
			frame = false,
			pointA = false,
			pointB = false,
			x = false,
			y = false,
			count = 1,
			keybinding = "",
			-- those appear on buttons and groups
			framestrata = "MEDIUM",
			popframestrata = "HIGH",
			left = 1,
			right = 2,
			middle = 3,
			scale = 1,
			popscale = 1,
			skin = false,
			gloss = false,
			backdrop = false,
			colors = {},
			poppoint = "top",
			padding = 0,
			offset = false,
			text = {
				hotkey = {
					hidden = false,
					font = "Friz Quadrata TT",
					size = 12,
					offsetx = -1,
					offsety = -2,
				},
				count = {
					hidden = false,
					font = "Friz Quadrata TT",
					size = 10,
					offsetx = 0,
					offsety = -21,
				},
			},
--			clickWhichSetting = 1,
			["*"] = {
				["*"] = {},
			},
		},
		groups = {
			["*"] = {
				framestrata = "MEDIUM",
				popframestrata = "HIGH",
				left = 1,
				right = 2,
				middle = 3,
				scale = 1,
				popscale = 1,
				skin = false,
				gloss = false,
				backdrop = false,
				colors = {},
				poppoint = "top",
				padding = 0,
				offset = false,
				text = {
					hotkey = {
						hidden = false,
						font = "Friz Quadrata TT",
						size = 12,
						offsetx = -1,
						offsety = -2,
					},
					count = {
						hidden = false,
						font = "Friz Quadrata TT",
						size = 10,
						offsetx = 0,
						offsety = -21,
					},
				},
			},
		},
	},
}

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New(self.name.."DB", defaultDB, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self:SetEnabledState(self.db.profile.enabled)

	self.options = options
	AceConfigRegistry:RegisterOptionsTable(self.name, options)
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	self:RegisterChatCommand(self.name, function(input)
		local trimmed = input and input:trim()
		if not trimmed or trimmed == "" then
			AceConfigDialog:Open(addon.name)
		else
			AceConfigCmd.HandleCommand(addon, addon.name, addon.name, trimmed ~= "help" and trimmed or "")
		end
	end)
	LibButtonFacade = LibButtonFacade or LibStub("LibButtonFacade", true)
	if LibButtonFacade then
		LibButtonFacade:RegisterSkinCallback(self.name, self.LibButtonFacade_Update, self)
	end
end

function addon:OnEnable()
	self.db.profile.enabled = true
	if not self.header then
		self.header = CreateFrame("Frame", self.name.."Header", UIParent, "SecureStateHeaderTemplate")
		self.header:SetPoint("CENTER", UIParent)
		self.header:Show()
		self.buttons = {}
	end
	for id = 1, self.db.profile.count do
		self:AddButtonOptions(id)
	end
	for group in pairs(addon.db.profile.groups) do
		self:AddGroup(nil, group)
	end
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	LibKeyBound.RegisterCallback(self, "LIBKEYBOUND_ENABLED")
	LibKeyBound.RegisterCallback(self, "LIBKEYBOUND_DISABLED")
	LibKeyBound.RegisterCallback(self, "LIBKEYBOUND_MODE_COLOR_CHANGED")
	
	addon:UpdateButtons()
end

function addon:OnDisable()
	self.db.profile.enabled = false
	self:UnregisterAllEvents()
	for id, button in pairs(self.buttons) do
		button:Store()
		self.buttons[id] = nil
	end
	for option, values in pairs(getOptions("groups")) do
		values.hidden = true
	end
end

function addon:OnProfileChanged()
	local enabled = self.db.profile.enabled
	if enabled ~= self:IsEnabled() then
		if enabled then
			self:Enable()
		else
			self:Disable()
		end
	else
		self:OnDisable()
		if enabled then
			self:OnEnable()
		end
	end
end

function addon:UpdateButtons()
	local db = self.db.profile
	for id, button in pairs(self.buttons) do
		button:Store()
		self.buttons[id] = nil
	end
	for id = 1, db.count do
		local button = self.Frame:Retrieve()
		self.buttons[id] = button
		button:Setup(id, db[id])
		button:SetLock(locked)
		if button:IsShown() then
			LibStickyFrames:SetFrameHidden(button)
		else
			button:Show()
		end
	end
	for button in addon.Frame:Iterate() do
		button:LoadPosition()
	end
end

function addon:LibButtonFacade_Update(skin, gloss, backdrop, group, button, colors)
	if not group or group == "nil" then return end
	local id = tonumber(group)
	local db = id and self.db.profile[id] or self.db.profile.groups[group]
	db.skin = skin
	db.gloss = gloss
	db.backdrop = backdrop
	db.colors = colors
end

function addon:SetLock(value, value2)
	if type(value) == "table" then value = value2 end
	locked = value and true or false
	if InCombatLockdown() then
		wasLocked = locked
		return
	end
	for id, button in ipairs(self.buttons) do
		button:SetLock(locked)
	end
	self.header:SetAttribute("state", locked and "0" or "-1")
	LibStickyFrames:SetGroup(not locked)
	if not locked then 
		LibKeyBound:Deactivate()
		Poppins.stickyMode = true
--		self:ColorPoppins()
	else
		Poppins.stickyMode = false
	end
	self:ColorPoppins()
end

function addon:IsLocked()
	if InCombatLockdown() then
		return wasLocked
	else
		return locked
	end
end

function addon:PLAYER_REGEN_DISABLED()
	if not locked then
		wasLocked = false
		self:SetLock(true)
		LibKeyBound:Deactivate()
	end
end

function addon:PLAYER_REGEN_ENABLED()
	if not wasLocked then
		wasLocked = true
		self:SetLock(false)
	end
end

function addon:AddButton()
	local db = self.db.profile
	db.count = db.count + 1
	local id = db.count
	addon:AddButtonOptions(id)
	AceConfigRegistry:NotifyChange(self.name)
	addon:UpdateButtons()
end

function addon:RemoveButton(key, id)
	if type(id) == "table" then
		id = id.arg
	end
	if not id then return end
	local db = self.db.profile
	for i=1, (db.count - 1) do
		if i >= id then
			db[i] = db[i + 1]
			i = i + 1
		end
		local settings = db[i]
		local frame = settings.frame
		if frame then
			if frame == id then
				local button = self.buttons[i]
				local s = button:GetEffectiveScale()
			
				settings.frame = false
				settings.point = false
				settings.x = button:GetLeft() * s
				settings.y = button:GetTop() * s
			elseif frame > id then
				settings.frame = settings.frame - 1
			end
		end
	end
	db[db.count] = nil
	getOptions("buttons").remove.values[db.count] = nil
	db.count = db.count - 1

	local values = getOptions("buttons", tostring(id)).addRemove.args.remove.values
	for i in ipairs(values) do
		values[i] = nil
	end
	AceConfigRegistry:NotifyChange(self.name)
	addon:UpdateButtons()
end

local pointValues = { top = "Up", bottom = "Down", right = "Right", left = "Left", center = "Around", }
local groupValues = { [""] = "None" }
local function strataRange(value)
	if type(value)=="string" then
		if value == "HIGH" then return 3
		elseif value == "DIALOG" then return 4
		elseif value == "TOOLTIP" then return 5
		elseif value == "MEDIUM" then return 2
		else return 1
		end
	else
		if value == 3 then return "HIGH"
		elseif value == 4 then return "DIALOG"
		elseif value == 5 then return "TOOLTIP"
		elseif value == 2 then return "MEDIUM"
		else return "LOW"
		end
	end
end
do
	local function get(key)
		local id = key.arg
		local option = key[#key]
		if option == "framestrata" or option == "popframestrata" then
			return strataRange(addon.db.profile[id][option])
		else
			return addon.db.profile[id][option]
		end
	end
	local function set(key, value)
		local id = key.arg
		local option = key[#key]
		if option == "framestrata" or option == "popframestrata" then
			value = strataRange(value)
		end
		addon.db.profile[id][option] = value
		addon:UpdateButtons()
	end
	local function func(key)
		local id = key.arg
		local db = addon.db.profile[id]
		db.frame = false
		db.pointA = false
		db.pointB = false
		db.x = false
		db.y = false
		addon:UpdateButtons()
	end
	local function disabled(key)
		local id = key.arg
		local option = key[#key]
		if option == "remove" then
			return addon.db.profile[id].count == 1
		elseif addon.db.profile[id].group then
			return true
		elseif option == "offset" then
			return addon.db.profile[id].poppoint ~= "center"
--		elseif option == "left" or option == "right" or option == "middle" then
--			return addon.db.profile[id].clickWhichSetting ~= 3
		end
	end
	local function hidden(key)
		local id = key.arg
		if not id then return end
		return addon.db.profile.count < id
	end
	local function groupSet(key, value)
		if not value or value == "" then
			value = false
		end
		local id = key.arg
		local option = key[#key]
		local db = addon.db.profile[id]
		db[option] = value
		if value then
			for gOption, gValue in pairs(addon.db.profile.groups[value]) do
				db[gOption] = gValue
			end
		end
		addon:UpdateButtons()
	end
	function addon:AddButtonOptions(id)
		if id > self.db.profile.count then return end
		getOptions("buttons").remove.values[id] = ("Button %d"):format(id)
		local optionArgs = getOptions("buttons")
		optionArgs[tostring(id)] = optionArgs[tostring(id)] or {
			name = ("Button %d"):format(id),
			desc = ("Button %d options"):format(id),
			type = "group",
			arg = id,
			childGroups = "tab",
			args = {
				addRemove = {
					name = "General",
					desc = "Add and Remove PopButtons",
					type = "group",
					arg = id,
					order = 1,
					args = {
						new = {
							name = "New PopButton",
							desc = "Creates a new PopButton",
							type = "execute",
							func = "AddPopButton",
							hidden = false,
							arg = id,
							order = 1,
						},
						remove = {
							name = "Remove PopButton",
							desc = "Removes a PopButton",
							type = "select",
							set = "RemovePopButton",
							values = {},
							disabled = disabled,
							hidden = false,
							arg = id,
							order = 2,
						},
						posReset = {
							name = "Reset Position",
							desc = "Reset the buttons position",
							type = "execute",
							func = func,
							hidden = false,
							arg = id,
							order = 4,
						},
						group = {
							name = "Group",
							desc = "Set button group",
							type = "select",
							get = get,
							set = groupSet,
							values = groupValues,
							hidden = false,
							arg = id,
							order = 6,
						},
					},
				},
				layout = {
					name = "Layout",
					desc = "Visual Options for the Button and its PopButtons",
					type = "group",
					arg = id,
					order = 2,
					args = {
--[[				keybinding = {
							name = "Button Keybinding",
							desc = "Sets the buttons keybinding",
							type = "keybinding",
							get = get,
							set = set,
							hidden = false,
							arg = id,
							order = 5,
						},
]]--
						header1 = {
							type = "header",
							name = "Button Options",
							desc = "Options for the Buttons of this Button",
							order = 1,
						},
						framestrata = {
							name = "Frame Strata",
							desc = "Sets the Buttons Frame Strata",
							type = "range",
							min = 1,
							max = 5,
							step = 1,
							get = get,
							set = set,
							disabled = disabled,
							arg = id,
							order = 8,
						},
						scale = {
							name = "Scale",
							desc = "Sets the buttons scale",
							type = "range",
							min = 0.1,
							max = 2,
							bigStep = 0.1,
							isPercent = true,
							get = get,
							set = set,
							hidden = false,
							disabled = disabled,
							arg = id,
							order = 9,
						},
						header4 = {
							type = "header",
							name = "PopButton Options",
							desc = "Options for the PopButtons of this Button",
							order = 10,
						},
						popframestrata = {
							name = "Frame Strata",
							desc = "Sets the PopButtons Frame Strata",
							type = "range",
							min = 1,
							max = 5,
							step = 1,
							get = get,
							set = set,
							disabled = disabled,
							arg = id,
							order = 11,
						},
						popscale = {
							name = "Scale",
							desc = "Sets the PopButtons scale",
							type = "range",
							min = 0.1,
							max = 2,
							bigStep = 0.1,
							isPercent = true,
							get = get,
							set = set,
							hidden = false,
							disabled = disabled,
							arg = id,
							order = 12,
						},
						padding = {
							name = "Padding",
							desc = "Sets the PopButtons padding",
							type = "range",
							min = -100,
							max = 100,
							bigStep = 1,
							get = get,
							set = set,
							hidden = false,
							disabled = disabled,
							arg = id,
							order = 13,
						},
						poppoint = {
							name = "Direction",
							desc = "Sets the direction the PopButtons should point to",
							type = "select",
							get = get,
							set = set,
							values = pointValues,
							hidden = false,
							disabled = disabled,
							arg = id,
							order = 14,
						},
						offset = {
							name = "Offset",
							desc = "Sets the popbuttons offset when point is set to 'Around'",
							type = "range",
							min = 0,
							max = 359,
							bigStep = 1,
							get = get,
							set = set,
							hidden = false,
							disabled = disabled,
							arg = id,
							order = 15,
						},
					},
				},
				text = {
					name = "Text",
					desc = "Text Options",
					type = "group",
					order = 5,
					get = function(info)
						local id = info.arg
--						local option = info[#info]		-- the leaf
--						local option1 = info[#info-1]	-- should be "hotkey" or "count"
--						local option2 = info[#info-2]	-- should be "text"
						return addon.db.profile[id][info[#info-2]][info[#info-1]][info[#info]]
					end,
					set = function(info, value)
						addon.db.profile[id][info[#info-2]][info[#info-1]][info[#info]] = value
						if info[#info-1] == "hotkey" then
							for frame in addon.Frame:Iterate() do
								frame.button:UpdateHotkeys()
							end
						elseif info[#info-1] == "count" then
							for frame in addon.Frame:Iterate() do
								frame.button:UpdateCount()
							end
						end
					end,
					args = {
						hotkey = {
							name = "Text: Hotkey",
							desc = "Text: Hotkey",
							type = "group",
							order = 1,
							inline = true,
							args = {
								hidden = {
									name = "Hide Hotkeys",
									desc = "Hide Hotkeys",
									type = "toggle",
									disabled = disabled,
									arg = id,
									order = 1,
								},
								font = {
									name = "Font",
									desc = "Sets the font.",
									type = "select",
									dialogControl = 'LSM30_Font',
									values = fonts,
									get = function(info)
										local id = info.arg
										return fonts[addon.db.profile[id].text.hotkey.font]
									end,
									set = function(info,v)
										local id = info.arg
										addon.db.profile[id].text.hotkey.font=fonts[v]
										for frame in addon.Frame:Iterate() do
											frame.button:UpdateHotkeys()
										end
									end,
									disabled = disabled,
									arg = id,
									order=2,
								},
								size = {
									name = "FontSize",
									desc = "Sets the fonts size.",
									type = "range",
									min = 8,
									max = 15,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 3,
								},
								offsetx = {
									name = "Offset: x",
									desc = "Offset: x",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 4,
								},
								offsety = {
									name = "Offset: y",
									desc = "Offset: y",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 5,
								},
							},
						},
						count = {
							name = "Text: Count & Macro",
							desc = "Text: Count & Macro",
							type = "group",
							order = 2,
							inline = true,
							args = {
								hidden = {
									name = "Hide Count",
									desc = "Hide Count",
									type = "toggle",
									disabled = disabled,
									arg = id,
									order = 1,
								},
								font = {
									name = "Font",
									desc = "Sets the font.",
									type = "select",
									dialogControl = 'LSM30_Font',
									values = fonts,
									get = function(info)
										local id = info.arg
										return fonts[addon.db.profile[id].text.count.font]
									end,
									set = function(info,v)
										local id = info.arg
										addon.db.profile[id].text.count.font=fonts[v]
										for frame in addon.Frame:Iterate() do
											frame.button:UpdateCount()
										end
									end,
									disabled = disabled,
									arg = id,
									order=2,
								},
								size = {
									name = "FontSize",
									desc = "Sets the fonts size.",
									type = "range",
									min = 8,
									max = 15,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 3,
								},
								offsetx = {
									name = "Offset: x",
									desc = "Offset: x",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 4,
								},
								offsety = {
									name = "Offset: y",
									desc = "Offset: y",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									disabled = disabled,
									arg = id,
									order = 5,
								},
							},
						},
					},
				},
				clickActions = {
					name = "Click Actions",
					desc = "Set actions for clicking on PopButtons",
					type = "group",
					arg = id,
					order = 16,
					args = {
					--[[
						clickWhichSetting = {
							name = "Use Setting",
							desc = "Select which setting should be used",
							type = "select",
							values = whichSettingValues,
							get = get,
							set = set,
							arg = id,
							order = 1,
						},
						]]--
						left = {
							name = "Left Button Action",
							desc = "Left Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = disabled,
							arg = id,
							values = buttonValues,
							order = 3,
						},
						right = {
							name = "Right Button Action",
							desc = "Right Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = disabled,
							arg = id,
							values = buttonValues,
							order = 4,
						},
						middle = {
							name = "Middle Button Action",
							desc = "Middle Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = disabled,
							arg = id,
							values = buttonValues,
							order = 5,
						},
					},
				},
			},
			hidden = hidden,
			order = id,
		}
		for popButtonID = 1, self.db.profile[id].count do
			addon:AddPopButtonOptions(id, popButtonID)
		end
	end
end

function addon:AddPopButton(buttonID)
	if type(buttonID) == "table" then
		buttonID = buttonID.arg
	end
	if not buttonID then return end
	local db = self.db.profile[buttonID]
	db.count = db.count + 1
	addon:AddPopButtonOptions(buttonID, db.count)
	AceConfigRegistry:NotifyChange(self.name)
	addon:UpdateButtons()
end

function addon:RemovePopButton(buttonID, id)
	if type(buttonID) == "table" then
		buttonID = buttonID.arg
	end
	if not buttonID or not id then return end
	local db = self.db.profile[buttonID]
	if db.count == 1 then return end
	local i = id + 1
	while i <= db.count do
		db[i - 1] = db[i]
		i = i + 1
	end
	db[db.count] = nil
	db.count = db.count - 1

	getOptions("buttons", tostring(buttonID)).addRemove.args.remove.values[id] = nil
	AceConfigRegistry:NotifyChange(self.name)
	addon:UpdateButtons()
end

function addon:AddPopButtonOptions(buttonID, id)
	local db = self.db.profile
	if buttonID > db.count or id > db[buttonID].count then return end
	getOptions("buttons", tostring(buttonID)).addRemove.args.remove.values[id] = ("PopButton %d"):format(id)
end

do
	local function set(key, value)
		local group = key[#key-2] -- from -1 cause of additional group layer
		local option = key[#key]
		if option == "framestrata" or option == "popframestrata" then
			value = strataRange(value)
		end
		addon.db.profile.groups[group][option] = value
		for id, values in ipairs(addon.db.profile) do
			if values.group == group then
				values[option] = value
			end
		end
		addon:UpdateButtons()
	end
	local function get(key)
		local group = key[#key-2] -- from -1 cause of additional group layer
		local option = key[#key]
		if option == "framestrata" or option == "popframestrata" then
			return strataRange(addon.db.profile.groups[group][option])
		else
			return addon.db.profile.groups[group][option]
		end
	end
	local function disabled(key)
		local group = key[#key-2] -- from -1 cause of additional group layer
		return addon.db.profile.groups[group].poppoint ~= "center"
	end
	function addon:AddGroup(key, group)
		if type(key) == "string" then
			group = key
		end
		if not group then return end
		local optionArgs = getOptions("groups")
		optionArgs[group] = optionArgs[group] or {
			name = ("Group %s"):format(group),
			desc = ("Button %s options"):format(group),
			type = "group",
			childGroups = "tab",
			args = {
				layout = {
					name = "Layout",
					desc = "Visual Options for the Button and its PopButtons of this Group",
					type = "group",
					order = 2,
					args = {
						header1 = {
							type = "header",
							name = "Group Button Options",
							desc = "Options for the Buttons of this Button",
							order = 1,
						},
						framestrata = {
							name = "Frame Strata",
							desc = "Sets the Frame Strata of this Groups Buttons",
							type = "range",
							min = 1,
							max = 5,
							step = 1,
							get = get,
							set = set,
							order = 3,
						},
						scale = {
							name = "Scale",
							desc = "Sets the Scale of this Groups Buttons",
							type = "range",
							min = 0.1,
							max = 2,
							bigStep = 0.1,
							isPercent = true,
							get = get,
							set = set,
							order = 4,
						},
						header4 = {
							type = "header",
							name = "Group PopButton Options",
							desc = "Options for the PopButtons of this Button Group",
							order = 6,
						},
						popframestrata = {
							name = "Frame Strata",
							desc = "Sets the Frame Strata of this Groups PopButtons ",
							type = "range",
							min = 1,
							max = 5,
							step = 1,
							get = get,
							set = set,
							order = 8,
						},
						popscale = {
							name = "Scale",
							desc = "Sets the Scale of this Groups PopButtons",
							type = "range",
							min = 0.1,
							max = 2,
							bigStep = 0.1,
							isPercent = true,
							get = get,
							set = set,
							order = 9,
						},
						padding = {
							name = "Padding",
							desc = "Sets the Padding of this Groups PopButtons",
							type = "range",
							min = -100,
							max = 100,
							bigStep = 1,
							get = get,
							set = set,
							order = 10,
						},
						poppoint = {
							name = "Direction",
							desc = "Sets the direction the groups popbuttons should point to",
							type = "select",
							get = get,
							set = set,
							values = pointValues or {},
							order = 12,
						},
						offset = {
							name = "Offset",
							desc = "Sets the groups popbuttons offset when point is set to 'Around'",
							type = "range",
							min = 0,
							max = 359,
							bigStep = 1,
							get = get,
							set = set,
							disabled = disabled,
							order = 13,
						},
					},
				},
				text = {
					name = "Text",
					desc = "Text Options",
					type = "group",
					order = 5,
					get = function(info)
						local option = info[#info]		-- the leaf
--						local option1 = info[#info-1]	-- should be "hotkey" or "count"
--						local option2 = info[#info-2]	-- should be "text"
--						local group = info[#info-3]	-- should be group
						if option == "font" then
							return fonts[addon.db.profile.groups[info[#info-3]][info[#info-2]][info[#info-1]][option]]
						else
							return addon.db.profile.groups[info[#info-3]][info[#info-2]][info[#info-1]][option]
						end
					end,
					set = function(info, value)
						local option = info[#info]		-- the leaf
						local group = info[#info-3]	-- should be group
						
						if option == "font" then
							value = fonts[value]
						end
						addon.db.profile.groups[group][info[#info-2]][info[#info-1]][option] = value
						
						for id, values in ipairs(addon.db.profile) do
							if values.group == group then
							values[info[#info-2]][info[#info-1]][info[#info]] = value
							end
						end

						if info[#info-1] == "hotkey" then
							for frame in addon.Frame:Iterate() do
								frame.button:UpdateHotkeys()
							end
						elseif info[#info-1] == "count" then
							for frame in addon.Frame:Iterate() do
								frame.button:UpdateCount()
							end
						end

					end,
					args = {
						hotkey = {
							name = "Text: Hotkey",
							desc = "Text: Hotkey",
							type = "group",
							order = 1,
							inline = true,
							args = {
								hidden = {
									name = "Hide Hotkeys",
									desc = "Hide Hotkeys",
									type = "toggle",
									order = 1,
								},
								font = {
									name = "Font",
									desc = "LibSharedMedia-3.0 (optional dependency) is required for this option.",
									type = "select",
									dialogControl = 'LSM30_Font',
									values = fonts,
									order=2,
								},
								size = {
									name = "FontSize",
									desc = "LibSharedMedia-3.0 (optional dependency) is required for this option.",
									type = "range",
									min = 8,
									max = 15,
									step = 1,
									order = 3,
								},
								offsetx = {
									name = "Offset: x",
									desc = "Offset: x",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									order = 4,
								},
								offsety = {
									name = "Offset: y",
									desc = "Offset: y",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									order = 5,
								},
							},
						},
						count = {
							name = "Text: Count & Macro",
							desc = "Text: Count & Macro",
							type = "group",
							order = 2,
							inline = true,
							args = {
								hidden = {
									name = "Hide Count",
									desc = "Hide Count",
									type = "toggle",
									order = 1,
								},
								font = {
									name = "Font",
									desc = "Font",
									type = "select",
									dialogControl = 'LSM30_Font',
									values = fonts,
									order=2,
								},
								size = {
									name = "FontSize",
									desc = "FontSize",
									type = "range",
									min = 8,
									max = 15,
									step = 1,
									order = 3,
								},
								offsetx = {
									name = "Offset: x",
									desc = "Offset: x",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									order = 4,
								},
								offsety = {
									name = "Offset: y",
									desc = "Offset: y",
									type = "range",
									min = -36,
									max = 0,
									step = 1,
									order = 5,
								},
							},
						},
					},
				},
				clickActions = {
					name = "Click Actions",
					desc = "Set actions for clicking on this groups PopButtons",
					type = "group",
					order = 16,
					args = {
						left = {
							name = "Left Button Action",
							desc = "Left Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = false,
							values = buttonValues,
							order = 3,
						},
						right = {
							name = "Right Button Action",
							desc = "Right Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = false,
							values = buttonValues,
							order = 4,
						},
						middle = {
							name = "Middle Button Action",
							desc = "Middle Button Action",
							type = "select",
							get = get,
							set = set,
							disabled = false,
							values = buttonValues,
							order = 5,
						},
					},
				},
			},
		}
		optionArgs[group].hidden = false
		groupValues[group] = group
		getOptions("groups").remove.values[group] = group
	end

	function addon:RemoveGroup(key, group)
		addon.db.profile.groups[group] = nil
		local optionArgs = getOptions("groups")
		if optionArgs[group] then
			optionArgs[group].hidden = true
		end
		groupValues[group] = nil
		optionArgs.remove.values[group] = nil
	end
end

-- LibKeyBound related functions
function addon:ColorPoppins()
	for button in addon.Frame:Iterate() do
		button:ColorBars()	
	end
end

function addon:LIBKEYBOUND_ENABLED()
	self:SetLock(true)
	Poppins.assignBindings = true
	Poppins:ColorPoppins()
end

function addon:LIBKEYBOUND_DISABLED()
	Poppins.assignBindings = nil
	Poppins:ColorPoppins()
end

function addon:LIBKEYBOUND_MODE_COLOR_CHANGED()
	Poppins:ColorPoppins()
end
